import os
import glob
import toml
import sys


questions = {
	"name":"Full name of project",
	"short_name":"Short name of project",
	"author":"Your name(s)",
	"email":"Project email",
	"short_description":"Short description of your project",
	"description":"Full description of your project",
	"url":"Main project URL",
	"license":"License",
	"keywords":"Keywords seperated with a space",
	"data_files":"Data files (with *) to include in project seperated with a space",
	"file":"Main python file, leave blank for no main file",
	"modules":"Required modules, seperated with space"
}
questions2= {
	"type":"Please choose one of the following options for your project:\n\t1. My project only deals with text (not graphical).\n\t2. My project is a graphical app.\n>>>",
	"mac":"Would you like to distribute a Mac app for your project? [y/n]",
	"windows":"Would you like to distribute a Windows app for your project? [y/n]",
}


def main():
	"""
	The setup wizard for shipsnake
	"""
	prefix = os.path.dirname(os.path.abspath(__file__))
	data = {}
	if os.path.isfile('.'+os.sep+'shipsnake.toml'):
		data = toml.loads(open('.'+os.sep+'shipsnake.toml').read())
	print("I'm going to ask you a few questions about your project to create a config file!\n\nPlease fill in everything to your ability. If you can't provide a value, leave it blank.\n")
	for key in questions:
		if key not in data:
			if len(glob.glob('.'+os.sep+'LICENS*'))>0 and key=="license":
				print('\033[1;34mYou seem to have a license file in your project, but what type is it?')
				for license in "AGPL-3.0/Apache-2.0/BSD-2-Clause/BSD-3-Clause/GPL-2.0/GPL-3.0/LGPL-2.1/LGPL-3.0/MIT".split(os.sep+''):
					print(f'\t- {license}')
				data[key] = input(">>>\033[0m ")
			elif len(glob.glob('.'+os.sep+'LICENSE*'))==0 and key=="license":
				data[key]=""
			elif key=="data_files" or key=="modules":
				data[key] = input("\033[1;34m"+questions[key]+":\033[0m ").split(' ')
			else:
				data[key] = input("\033[1;34m"+questions[key]+":\033[0m ")
		else:
			print("\033[1;34m"+str(questions[key])+":\033[0m "+str(data[key]))
	if "build" not in data:
		data['build']={}
	
	for key in questions2:
		if key not in data['build']:
			c=input("\033[1;34m"+questions2[key]+"\033[0m ")
			data['build'][key] = c[0]=='y' if '[y/n]' in questions2[key] else c
		else:
			print("\033[1;34m"+str(questions2[key])+"\033[0m "+str(data['build'][key]))
	if (data['build']['mac'] or data['build']['windows']):
		if 'installer' in data['build']:
			print("\033[0m"+"Would you like to provide an installer for the app(s)? (reccomended) [y/n]"+"\033[0m "+str(data['build']['installer']))
		else:
			data['build']['installer'] = True if input("\033[1;34m"+"Would you like to provide an installer for the app(s)? (reccomended) [y/n]"+"\033[0m ")[0]=="y" else False

	if not sys.platform.startswith('win') and data['build']['windows']:
		print('\033[1;31mYou can only make a Windows app on a Windows computer.\033[0m')
		needswin=True
	else:
		needswin=False
	if sys.platform != 'darwin' and data['build']['mac']:
		print('\033[1;31mYou can only make a Mac app on a Mac.\033[0m')
		needsmac=True
	else:
		needsmac=False

	if (needsmac or needswin):
		if ('actions' in data['build']):
			print('\033[1;34mGithub Actions can build your app on the cloud for free.\n\033[1;31mWould you like to use that to build your apps? [y/n] \033[0m'+str(data['build']['actions']))
			actions = data['build']['actions']
		else:
			actions = True if input('\033[1;34mGithub Actions can build your app on the cloud for free.\n\033[1;31mWould you like to use that to build your apps? [y/n] \033[0m')[0] == 'y' else False
			data['build']['actions'] = actions
	else:
		actions = False
	if actions:
		try:
			f = open('.github'+os.sep+'workflows'+os.sep+'shipsnake.yml','w+')
		except:
			os.system('mkdir -p .github'+os.sep+'workflows'+os.sep)
			f = open('.github'+os.sep+'workflows'+os.sep+'shipsnake.yml','w+')
		f.write(open(prefix+os.sep+'shipsnake.yml.template').read())
		if 'github' not in data['build']:
			ghrepo = True if input('\033[1;34mDo you have a GitHub repo for this project yet? [y/n] \033[0m')[0] == 'y' else False
			if not ghrepo:
				create = True if input('\033[1;34mWould you like me to create one? [y/n] \033[0m')[0] == 'y' else False
				if create:
					print('\033[1;31mGo to https://github.com/new and create a repo called `'+data['short_name']+'`. When you\'ve done that, come back here and push enter.\033[0m')
					input('')
					ghusername = input("\033[1;34mYour github username: \033[0m")
					data['build']['github'] = ghusername+"/"+data['short_name']
					os.system(f'git init;git config user.name "{data["author"]}";git config user.email "{data["email"]}";git add .;git commit -m ":rocket: Initial Commit!";git remote add origin https://github.com/{ghusername}/{data["short_name"]}.git;git push -u origin master;')
				else:
					print('\033[1;31mOk!\033[0m')
					data['build']['github'] = 'N/A'
					print('\033[1;31m\nI have created the neccesary files in your project. Please upload this folder to GitHub to use this feature.\n\n\033[0mFor help, see https://docs.github.com/en/free-pro-team@latest/github/importing-your-projects-to-github/adding-an-existing-project-to-github-using-the-command-line\033[0m')
			else:
				data['build']['github'] = input('\033[1;34mWhat is your repo? (user/repo): \033[0m')
				print('\033[1;31m\nI have created the neccesary files in your project. Please upload this folder to GitHub to use this feature.\n\n\033[0mFor help, see https://docs.github.com/en/free-pro-team@latest/github/importing-your-projects-to-github/adding-an-existing-project-to-github-using-the-command-line\033[0m')
		else:
			print('\033[1;34mDo you have a GitHub repo for this project yet? [y/n] \033[0my')
	open('README.md','w+').write(open(prefix+os.sep+'readme.template').read().format(**data))
	if 'bundle_id' not in data:
		data['bundle_id'] = f"com.{data['author'].lower().replace(' ','')}.{data['short_name']}"
	if 'homebrew' not in data:
		data['homebrew'] = False
	if 'icon' not in data:
		data['icon'] = "False"
	
	f = open('.'+os.sep+'shipsnake.toml','w+')
	f.write(toml.dumps(data))
	f.close()

	print('\n\n\033[1;31m`shipsnake.toml` has been created. Please edit that for further changes and advanced options.\033[0m')


import os
import sys
import toml

def main(arguments,ids):
	if not os.path.isfile('.'+os.sep+'shipsnake.toml'):
		print('Please create a config file with `shipsnake wizard` first.')
		sys.exit(0)
	with open('.'+os.sep+'shipsnake.toml') as datafile:
		data = toml.loads(datafile.read())
	version = data['latest_build']
	if "pypi" in ids or len(ids)==1:
		print("Please make sure that you have a https://pypi.org/ account.")
		try:
			import twine
		except:
			input('Press enter to continue installing `twine`. Press ctrl+c to exit.')
			os.system('python3 -m pip install --user --upgrade twine || python3 -m pip install --upgrade twine')
			import twine
		del twine
		os.system('python3 -m twine upload dist'+os.sep+'pypi'+os.sep+'*')
	if "github" in ids or len(ids)==1:
		f = f'git config user.name "{data["author"]}";git config user.email "{data["email"]}";git add .;git tag v{version};git commit -m "Release v{version}";git remote add origin https://github.com/{data["build"]["github"]}.git || echo;git push -u origin master --tags;'
		os.system(f)


import os
import sys
import toml
import glob
import shutil

prefix = os.path.dirname(os.path.abspath(__file__))

def main(version,arguments,nointeraction=False):
	# ============== Get Data ===============================================
	if not os.path.isfile('.'+os.sep+'shipsnake.toml'):
		print('Please create a config file with `shipsnake wizard` first.')
		sys.exit(0)
	with open('.'+os.sep+'shipsnake.toml') as datafile:
		data = toml.loads(datafile.read())

	# ============== Show what will happen ===============================================
	print('\t- Generate a correct directory structure, setup.py, .gitignore, etc...')
	only=False
	for x in arguments:
		if '-only' in x[0]:
			only=True
			break
	dopypi,dobrew,domac,dowin,doact=False,False,False,False,False
	if only:
		if ('--pypi-only','') in arguments:
			dopypi = True
		elif ('--homebrew-only','') in arguments:
			dobrew = True
		elif ('--windows-only','') in arguments and sys.platform.startswith('win'):
			dowin = True
		elif ('--mac-only','') in arguments and sys.platform.startswith('darwin'):
			domac = True
		elif ('--actions-only','') in arguments:
			doact = True
	else:
		dopypi = True
		dobrew = data['build']['homebrew']
		dowin = data['build']['windows'] and sys.platform.startswith('win')
		domac = data['build']['mac'] and sys.platform.startswith('darwin')
		doact= data['build']['actions']
	if ('--no-installer','') in arguments:
		doinstall=False
	else:
		doinstall=data['build']['installer']
	if dopypi:
		print('\t- A distributable Python module.')
	if dobrew:
		print('\t- A Homebrew package.')
	if dowin:
		installer = " with a .msi installer." if doinstall else "."
		print('\t- A Windows app'+installer)
	if domac:
		installer = " with a .dmg installer." if doinstall else "."
		print('\t- A Mac app'+installer)
	if doact:
		print('\t- A GitHub Actions file for building Mac and Windows apps and publishing a Github release.')
	if not nointeraction:
		input('Press enter to continue.')
	# ============== Create bin Script ===============================================
	try:
		os.mkdir('bin')
	except:
		pass
	open("bin"+os.sep+data['short_name'],'w+').write(f"#!"+os.sep+"usr"+os.sep+"bin"+os.sep+f"env bash\npython3 -m {data['short_name']} $@")

	# ============== Generate setup.py ===============================================
	with open(prefix+os.sep+'setup.py.template') as datafile:
		template = datafile.read()
	setup = template.format(
		**data,
		version = version,
		entry_points = [data["short_name"]+"="+data["short_name"]+".__main__"] if data["file"]!="" else [""]
	)
	open('setup.py','w+').write(setup)

	# ============== Generate directory structure ===============================================
	open('.'+os.sep+'.gitignore','w+').write(open(prefix+os.sep+'gitignore.template').read().replace('/',os.sep))
	source_dir = os.getcwd()
	target_dir = data["short_name"]+os.sep
	types = ('*.py',*data["data_files"])
	file_names = []
	for files in types:
		file_names.extend(glob.glob(files))
	if not os.path.isdir(target_dir):
		os.mkdir(target_dir)
	for file_name in file_names:
		if file_name in ["setup.py","shipsnake.toml"]:
			continue
		shutil.move(os.path.join(source_dir, file_name), target_dir+os.sep+file_name)
	for filename in glob.glob(target_dir+os.sep+'LICE*'):
		shutil.copyfile(filename,'LICENSE')
	open(target_dir+'__init__.py','w+').write('# This file must exist, empty or not')
	if data['file']!="" and not os.path.isfile(data['short_name']+os.sep+'__main__.py'):
		try:
			os.rename(data['short_name']+os.sep+data['file'],data['short_name']+os.sep+'__main__.py')
			open(data['short_name']+os.sep+data['file'],'w+').write('# Please edit __main__.py for the main code. Thanks!\n(you can delete this file.)')
		except FileNotFoundError:
			pass


	# ============== Generate pypi files ===============================================
	if dopypi:
		try:
			shutil.rmtree('dist')
		except:
			pass
		os.system('python3 .'+os.sep+'setup.py sdist -d dist'+os.sep+'pypi bdist_wheel')
		try:
			shutil.rmtree('build')
		except:
			pass
		for x in glob.glob('*.egg-info'):
			shutil.rmtree(x)
		for x in glob.glob('dist'+os.sep+'*.whl'):
			os.rename(x,x.replace('dist'+os.sep,'dist'+os.sep+'pypi'+os.sep))

	# ============== Generate w/Pyinstaller ===============================================
	if dowin or domac:
		try:
			import PyInstaller.__main__
		except:
			print('Installing PyInstaller...')
			os.system('pip3 install pyinstaller')
			import PyInstaller.__main__
		
		mods = []
		for x in data['modules']:
			mods.append('--hidden-import')
			mods.append(x)

		datafiles = []
		for x in data['data_files']:
			for g in glob.glob(data['short_name']+os.sep+x):
				datafiles.append('--add-data')
				datafiles.append(g+os.pathsep+g.replace(data['short_name']+os.sep,''))
		typ = '--nowindowed' if data['build']['type']=='1' else '--noconsole'

		ico = ['--icon',data['icon']] if 'icon' in data else []
		options = [
			data['short_name']+os.sep+'__main__.py',
			'--onefile',
			'--name',
			data['name'] if ('-n','')not in arguments else 'app',
			'--distpath',
			'.'+os.sep+'dist'+os.sep+'pyinstaller',
			*mods,
			*datafiles,
			typ,
			*ico,
			'--osx-bundle-identifier',
			data['bundle_id']
		]
		print(options)
		PyInstaller.__main__.run(options)
		print('removing '+data['name']+".spec...")
		os.remove(data['name']+".spec")
	# ============== Generate Installer Package ===============================================

	if not doinstall:
		pass
	elif sys.platform.startswith('win'):#WINDOWS
		print('Installer creation not yet supported on Windows.')
	elif sys.platform.startswith('darwin'):#MAC
		try:
			import dmgbuild
		except:
			print('Installing dmgbuild')
			os.system('pip3 install dmgbuild')
			import dmgbuild
		del dmgbuild
		open('build'+os.sep+'settings.py','w+').write(open(prefix+os.sep+'settings.py.template').read().format(
			**data,
			version=version,
			icns=data['icon']
		))
		os.system(f'cat build/settings.py;dmgbuild -s .{os.sep}build{os.sep}settings.py "{data["name"]}" app.dmg')

	else:
		print(f'Installer creation not yet supported for {sys.platform}!')
	# ============== Generate Github Actions Workflow ===============================================
	if doact:
		try:
			f = open('.github'+os.sep+'workflows'+os.sep+'shipsnake.yml','w+')
		except:
			os.system('mkdir -p .github'+os.sep+'workflows'+os.sep)
			f = open('.github'+os.sep+'workflows'+os.sep+'shipsnake.yml','w+')
		f.write(open(prefix+os.sep+'shipsnake.yml.template').read())
	# ============== Save Version ===============================================
	print('saving version...')
	data['latest_build'] = version
	open('shipsnake.toml','w+').write(toml.dumps(data))


if __name__ != '__main__':
	print("Please run shipsnake as script or command, not as a module.")

import sys
import getopt
import os

he = "Provide a mode:\n\tshipsnake [wizard | build | dev | upload]"

try:
	arguments, ids = getopt.getopt(sys.argv[1:], "y", ['pypi-only','pyinstaller-only','homebrew-only','actions-only','windows-only','mac-only'])
except:
	print(he)
	sys.exit(0)

noint = ('-y','') in arguments

try:
	mode = ids[0]
except:
	print(he)
	sys.exit(0)

try:
	version = ids[1]
except:
	version = ''

if version=='' and mode in ["build"]:
	print("Provide a version:\n\tshipsnake "+mode+" <version>")
	sys.exit(0)

if mode=="dev" and version=="":
	version = "dev_build"

# ================================================================

if mode == "wizard":
	import shipsnake.wizard as wizard
	wizard.main()

elif mode == "build":
	print('This command will build the following:')
	import shipsnake.build as build
	build.main(version,arguments,nointeraction=noint)

elif mode == "upload":
	import shipsnake.upload as upload
	upload.main(arguments,ids)
# elif mode in ["build","dev","upload"]:


# 	elif mode == "dev":
# 		os.system('python3 .'+os.sep+'setup.py develop')
# 	for x in glob.glob('*.egg-info'):
# 		shutil.rmtree(x)
# else:
# 	print(f'Illegeal option `{mode}`')
# 	sys.exit(0)

# if mode=="upload":
# 	print("Please make sure that you have a https://pypi.org/ account.")
# 	try:
# 		import twine
# 	except:
# 		input('Press enter to continue installing `twine`. Press ctrl+x to exit.')
# 		os.system('python3 -m pip install --user --upgrade twine || python3 -m pip install --upgrade twine')
# 	os.system('python3 -m twine upload dist'+os.sep+'*')
